var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import childProcess from 'child_process';
import path from 'node:path';
import { fileURLToPath } from 'node:url';
import chalk from 'chalk';
import spawn from 'cross-spawn';
import fs from 'fs-extra';
import inquirer from 'inquirer';
import semver from 'semver';
import { deleteExistFileQuestion } from '../constants/questions.js';
const execSync = childProcess.execSync;
// https://flaviocopes.com/fix-dirname-not-defined-es-module-scope/
const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);
export const hasProperty = (target, key) => 
// eslint-disable-next-line prefer-object-has-own
Object.prototype.hasOwnProperty.call(target, key);
export const checkCurrentNodeVersion = (wanted) => {
    if (!semver.satisfies(process.version, wanted)) {
        console.log(chalk.red('Your current Node version is ' +
            process.version +
            ', but the cli need' +
            wanted +
            'version.'));
        process.exit(1);
    }
};
export const getPackageInfo = () => {
    return JSON.parse(fs.readFileSync(path.resolve(__dirname, '../../package.json')));
};
/**
 * @description 安装依赖包
 * @param {dependencies} 依赖
 * @param {isDev} 是否为开发依赖
 * @param {cwd} 安装路径
 * @date 2021-05-03 21:17:48
 */
export const installPackages = ({ dependencies = [], isDev = true, cwd = '', stdio = 'inherit' }) => __awaiter(void 0, void 0, void 0, function* () {
    if (dependencies.length === 0)
        return;
    const { usePnpm, useYarn, useNpm, allPassed } = yield checkPackageTools();
    if (!allPassed) {
        console.log(chalk.red('Please install npm or yarn'));
        process.exit(1);
    }
    // 检测是否存在package.json 文件
    const packagePath = path.join(cwd, 'package.json');
    if (!fs.existsSync(packagePath)) {
        console.log(chalk.yellow('creating package.json file..'));
        yield packageInit(cwd);
    }
    try {
        let command;
        let args;
        if (usePnpm) {
            command = 'pnpm';
            args = 'add';
            dependencies.length > 0 && (args = ['add', '--exact']);
            isDev && args.push('--dev');
        }
        else if (useYarn) {
            command = 'yarnpkg';
            // 这里区分两种情况，1. 无依赖 只需要执行 yarn install 2.有依赖区分是开发还是生产
            args = 'install';
            dependencies.length > 0 && (args = ['add', '--exact']);
            isDev && args.push('--dev');
        }
        else {
            command = 'npm';
            args = ['install'];
            isDev && args.push('--save-dev');
        }
        dependencies.length > 0 && [].push.apply(args, dependencies);
        const child = spawn.sync(command, args, { stdio, cwd });
        if (child.status !== 0) {
            console.log(chalk.red(`\`${command} ${args.join(' ')}\` failed`));
        }
    }
    catch (e) {
        console.log(chalk.yellow(e.message || '安装依赖失败'));
        process.exit(1);
    }
});
/**
 * @description 删除包
 * @date 2021-05-04 11:07:36
 */
export const removePackages = ({ dependencies = [], cwd = '', stdio = 'inherit' }) => __awaiter(void 0, void 0, void 0, function* () {
    if (dependencies.length === 0)
        return;
    const { usePnpm, useYarn, useNpm, allPassed } = yield checkPackageTools();
    if (!allPassed) {
        console.log(chalk.red('Please install pnpm or npm or yarn'));
        process.exit(1);
    }
    try {
        let command;
        let args = ['remove'];
        if (usePnpm) {
            command = 'pnpm';
            args = ['remove'];
        }
        else if (useYarn) {
            command = 'yarnpkg';
            args = ['remove'];
        }
        else {
            command = 'npm';
            args = ['uninstall'];
        }
        ;
        [].push.apply(args, dependencies);
        const child = spawn.sync(command, args, { stdio, cwd });
        if (child.status !== 0) {
            console.error(`\`${command} ${args.join(' ')}\` failed`);
        }
    }
    catch (e) {
        console.log();
    }
});
export const canUsePnpm = () => {
    return new Promise((resolve, reject) => {
        try {
            execSync('pnpm --version', { stdio: 'ignore' });
            resolve(true);
        }
        catch (e) {
            resolve(false);
        }
    });
};
export const canUseYarn = () => {
    return new Promise((resolve, reject) => {
        try {
            execSync('yarnpkg --version', { stdio: 'ignore' });
            resolve(true);
        }
        catch (e) {
            resolve(false);
        }
    });
};
export const canUseNpm = () => {
    return new Promise((resolve, reject) => {
        try {
            execSync('npm --version', { stdio: 'ignore' });
            resolve(true);
        }
        catch (e) {
            resolve(false);
        }
    });
};
/**
 * @description 检查是否能用yarn or npm or pnpm
 * @param {*} async
 * @date 2021-05-04 11:18:52
 * @see
 * @return {*}
 */
export const checkPackageTools = () => __awaiter(void 0, void 0, void 0, function* () {
    const [usePnpm, useYarn, useNpm] = yield Promise.all([canUsePnpm(), canUseYarn(), canUseNpm()]);
    const allPassed = usePnpm && useYarn && useNpm;
    return {
        usePnpm,
        useYarn,
        useNpm,
        allPassed
    };
});
/**
 * @description 包初始化 执行命令为 npm init -y or yarn init -y
 * @param {cwd} 执行路径
 * @date 2021-05-04 22:10:27
 */
export const packageInit = (cwd = '') => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { useYarn = false, useNpm = false, usePnpm = false, allPassed } = yield checkPackageTools();
        let command;
        const args = ['init', '--yes'];
        if (!allPassed) {
            console.log(chalk.red('Please install pnpm or yarn or npm!'));
            process.exit(1);
        }
        else if (usePnpm) {
            command = 'pnpm';
        }
        else if (useYarn) {
            command = 'yarnpkg';
        }
        else {
            command = 'npm';
        }
        const child = spawn.sync(command, args, { stdio: 'ignore', cwd });
        if (child.status !== 0) {
            console.error(`\`${command} ${args.join(' ')}\` failed`);
            return true;
        }
        return false;
    }
    catch (e) {
        return false;
    }
});
/**
 * @description 检查文件是否存在
 * @param {string} files
 * @param {string} cwd
 * @return {string} filename
 * @date 2021-06-14 16:47:04
 */
export const checkFileIfExists = (files, cwd = process.cwd()) => {
    if (!files.length)
        return undefined;
    return files.find((file) => {
        const filePath = path.resolve(cwd, file);
        const result = fs.pathExistsSync(filePath);
        return fs.pathExistsSync(filePath);
    });
};
/**
 * @description 检查路径是否存在(文件夹)
 * @param {string} filePath
 * @param {*} canDelete 能否被删除
 * @date 2022-04-23 14:36:01
 * @return {Boolean}
 */
export const filePathExist = (filePath, canDelete = false) => __awaiter(void 0, void 0, void 0, function* () {
    if (fs.existsSync(filePath)) {
        if (canDelete) {
            const { toBeDeleted } = yield inquirer.prompt(deleteExistFileQuestion);
            if (toBeDeleted) {
                fs.removeSync(filePath);
                console.log(chalk.blue(`The file path:${filePath} is deleted successfully`));
                return false;
            }
            else {
                console.log(chalk.red(`The file path:${filePath} already exists`));
                return process.exit(1);
            }
        }
        console.log(chalk.red(`The file path:${filePath} already exists`));
        return process.exit(1);
    }
    else {
        return false;
    }
});
/**
 * @description 复制文件
 * @date 2022-11-13 16:03:27
 */
export const copyFile = (source, dest) => {
    return fs.copySync(source, dest, {
        overwrite: false
    });
};
/**
 * @description 读json文件
 * @param {string} jsonFilePath
 * @date 2022-10-11 16:30:18
 * @return {*} 返回解析后的json格式
 */
export const readJson = (jsonFilePath) => {
    if (!jsonFilePath)
        return;
    return JSON.parse(fs.readFileSync(jsonFilePath));
};
/**
 * @description 将对象输出为json文件
 * @param {string} writePath
 * @param {any} writeData
 * @date 2022-10-11 16:34:49
 * @return {*}
 */
export const writeJson = ({ writePath, writeData, cover = true }) => {
    const existed = filePathExist(writePath, true);
    if (!existed && !cover) {
        throw new Error(`输出Json文件失败，${writePath}已存在`);
    }
    return fs.writeFileSync(writePath, JSON.stringify(writeData, null, 2), 'utf-8');
};
/**
 * @description 根据key值合并对象
 * @param {*} originObj
 * @param {*} targetObj
 * @param {*} mergeKeys
 * @date 2022-11-13 15:41:51
 * @return {AnyOptions}
 */
export const mergeObject = ({ originObj, targetObj, mergeKeys = [] }) => {
    if (!mergeKeys || !originObj || !targetObj)
        return;
    const newObj = mergeKeys.reduce((obj, key) => {
        if (!hasProperty(originObj, key) || !hasProperty(targetObj, key))
            return obj;
        return Object.assign(Object.assign({}, targetObj), { [key]: Object.assign(Object.assign({}, targetObj[key]), originObj[key]) });
    }, {});
    return newObj;
};
